@@ -27,6 +27,7 @@
 #
 import sys
 import os
+import json
 import logging
 import shutil
 import stat
@@ -142,20 +143,20 @@ printenv | sort
 pwd
 ls -lsa
 
-if [ "\${SITE_EXPECT_HOME_DIR}" = "" -o "\${SITE_EXPECT_HOME_DIR}" = "YES" ] ; then
-   if [ "\${HOME}" != "" ] ; then
-      cd \${HOME}
-      exitStatus=\$?
+if [ "\\${SITE_EXPECT_HOME_DIR}" = "" -o "\\${SITE_EXPECT_HOME_DIR}" = "YES" ] ; then
+   if [ "\\${HOME}" != "" ] ; then
+      cd \\${HOME}
+      exitStatus=\\$?
 
-      if [ \$exitStatus -eq 0 ] ; then
+      if [ \\$exitStatus -eq 0 ] ; then
          quota -s
-         exitStatus=\$?
-         if [ \$exitStatus -ne 0 ] ; then
+         exitStatus=\\$?
+         if [ \\$exitStatus -ne 0 ] ; then
             quota
-            exitStatus=\$?
-            if [ \$exitStatus -ne 0 ] ; then
+            exitStatus=\\$?
+            if [ \\$exitStatus -ne 0 ] ; then
                du -sh
-               exitStatus=\$?
+               exitStatus=\\$?
             fi
          fi
       else
@@ -164,7 +165,7 @@ if [ "\${SITE_EXPECT_HOME_DIR}" = "" -o "\${SITE_EXPECT_HOME_DIR}" = "YES" ] ; t
    fi
 fi
 
-exit \$exitStatus
+exit \\$exitStatus
 EOF
          chmod +x probegridsite.sh
       fi
@@ -407,9 +408,9 @@ exit ${exitStatus}
    def configure(self):
       configured = False
 
-      sectionPattern  = re.compile('(\s*\[)([^\s]*)(]\s*)')
-      keyValuePattern = re.compile('( *)(\w*)( *= *)(.*[^\s$])( *)')
-      commentPattern  = re.compile('\s*#.*')
+      sectionPattern  = re.compile(r'(\s*\[)([^\s]*)(]\s*)')
+      keyValuePattern = re.compile(r'( *)(\w*)( *= *)(.*[^\s$])( *)')
+      commentPattern  = re.compile(r'\s*#.*')
       inProbeSection  = False
 
       configured = False
@@ -454,20 +455,27 @@ exit ${exitStatus}
                                  sampleKey   = list(self.configData[key].keys())[0]
                                  sampleValue = self.configData[key][sampleKey]
                               except:
-                                 sampleKey   = "key"
-                                 sampleValue = "value"
-                              self.configData[key] = {} 
-                              for e in value.split(','):
-                                 dictKey,dictValue = e.split(':')
-                                 if isinstance(sampleKey,int):
-                                    dictKey = int(dictKey)
-                                 if   isinstance(sampleValue,int):
-                                    dictValue = int(dictValue)
-                                 elif isinstance(sampleValue,float):
-                                    dictValue = float(dictValue)
-                                 elif isinstance(sampleValue,bool):
-                                    dictValue = bool(dictValue.lower() == 'true')
-                                 self.configData[key][dictKey] = dictValue
+                                 try:
+                                    self.configData[key] = json.loads(value)
+                                 except:
+                                    self.configData[key] = {}
+                                    sampleKey   = "key"
+                                    sampleValue = "value"
+                              else:
+                                 self.configData[key] = {}
+               
+                              if not self.configData[key]:
+                                 for e in value.split(','):
+                                    dictKey,dictValue = e.split(':')
+                                    if isinstance(sampleKey,int):
+                                       dictKey = int(dictKey)
+                                    if   isinstance(sampleValue,int):
+                                       dictValue = int(dictValue)
+                                    elif isinstance(sampleValue,float):
+                                       dictValue = float(dictValue)
+                                    elif isinstance(sampleValue,bool):
+                                       dictValue = bool(dictValue.lower() == 'true')
+                                    self.configData[key][dictKey] = dictValue
                            else:
                               self.configData[key] = value
                         else:
@@ -528,9 +536,9 @@ exit ${exitStatus}
       del self.probeSites
       self.probeSites = {}
 
-      sitePattern     = re.compile('(.*\[)(.*)(])')
-      keyValuePattern = re.compile('( *)(\w*)( *= *)(.*[^\s$])( *)')
-      commentPattern  = re.compile('\s*#.*')
+      sitePattern     = re.compile(r'(.*\[)(.*)(])')
+      keyValuePattern = re.compile(r'( *)(\w*)( *= *)(.*[^\s$])( *)')
+      commentPattern  = re.compile(r'\s*#.*')
       siteName        = ""
 
       fpInfo = open(self.probeSitesPath,'r')
@@ -627,6 +635,32 @@ exit ${exitStatus}
             fpDumpFile.close()
 
 
+   def __writeToStdout(self,
+                       message):
+      try:
+         try:
+            sys.stdout.write(message)
+         except UnicodeEncodeError:
+            sys.stdout.write(message.encode('utf-8'))
+         sys.stdout.flush()
+      except IOError as e:
+         if not e.args[0] in [EPIPE]:
+            self.logger.log(logging.ERROR,getLogMessage("Can't write to stdout: %s" % (message)))
+
+
+   def __writeToStderr(self,
+                       message):
+      try:
+         try:
+            sys.stderr.write(message)
+         except UnicodeEncodeError:
+            sys.stderr.write(message.encode('utf-8'))
+         sys.stderr.flush()
+      except IOError as e:
+         if not e.args[0] in [EPIPE]:
+            self.logger.log(logging.ERROR,getLogMessage("Can't write to stderr: %s" % (message)))
+
+
    def executeProbeCommand(self,
                            command,
                            environment=None,
@@ -684,8 +718,7 @@ exit ${exitStatus}
                outEOF = True
             outData.append(outChunk)
             if streamOutput:
-               sys.stdout.write(outChunk)
-               sys.stdout.flush()
+               self.__writeToStdout(outChunk)
 
          if childerrFd in ready[0]:
             errChunk = os.read(childerrFd,self.bufferSize).decode('utf-8')
@@ -693,8 +726,7 @@ exit ${exitStatus}
                errEOF = True
             errData.append(errChunk)
             if streamOutput:
-               sys.stderr.write(errChunk)
-               sys.stderr.flush()
+               self.__writeToStderr(errChunk)
 
          if outEOF and errEOF:
             break
@@ -902,6 +934,7 @@ exit ${exitStatus}
                   except KeyError as e:
                      probeSiteShellScript = ""
                      self.logger.log(logging.ERROR,getLogMessage("Pattern substitution failed for @@{%s}\n" % (e.args[0])))
+                     self.logger.log(logging.ERROR,getLogMessage("in rawProbeSiteShellTemplate\n"))
                   except TypeError:
                      probeSiteShellScript = ""
                      self.logger.log(logging.ERROR,getLogMessage("Submission script substitution failed:\n%s\n" % \
