@@ -80,8 +80,11 @@ class SubmitCommand:
       self.versionDistributor   = False
       self.debug                = False
       self.local                = False
+      self.asynchronous         = False
       self.queryJobs            = []
       self.killJobs             = []
+      self.jsonFiles            = []
+      self.showCommand          = False
       self.venueStatus          = ""
       self.venues               = []
       self.inputFiles           = []
@@ -91,7 +94,10 @@ class SubmitCommand:
       self.separators           = []
       self.nCores               = None
       self.ppn                  = None
+      self.nGpus                = None
+      self.gpn                  = None
       self.nStripes             = None
+      self.memoryMB             = None
       self.wallTime             = None
       self.environmentVariables = {}
       self.runName              = ""
@@ -446,6 +452,47 @@ class SubmitCommand:
          self.local = self.savedCommand['local']
 
 
+   def setAsynchronous(self,
+                       asynchronous=False):
+      """
+      Turn asynchronous submit on or off.
+      """
+      setError = False
+      if isinstance(asynchronous,bool):
+         self.asynchronous = asynchronous
+      else:
+         self.logger.log(logging.ERROR,getLogMessage("asynchronous must be a boolean"))
+         setError = True
+
+      return(setError)
+
+
+   def resetAsynchronous(self):
+      """
+      Turn asynchronous submit off.
+      """
+      resetError = False
+      self.asynchronous = False
+
+      return(resetError)
+
+
+   def __addAsynchronousArg(self):
+      if self.asynchronous:
+         self.args += ['--asynchronous']
+
+
+   def __saveAsynchronous(self):
+      if self.asynchronous:
+         self.savedCommand['asynchronous'] = self.asynchronous
+
+
+   def __loadAsynchronous(self):
+      self.resetAsynchronous()
+      if 'asynchronous' in self.savedCommand:
+         self.asynchronous = self.savedCommand['asynchronous']
+
+
    def setQueryJobs(self,
                     queryJobs):
       """
@@ -944,7 +991,7 @@ class SubmitCommand:
       Set number of cores per node requested for submit run.
       If number of cores per node is not set for the submit run
       a default number of cores per node is set based on the
-      venue configuration.  This default value is typical based
+      venue configuration.  This default value is typically based
       on the node hardware.
       """
       setError = False
@@ -983,6 +1030,92 @@ class SubmitCommand:
          self.setPPN(self.savedCommand['ppn'])
 
 
+   def setNgpus(self,
+                nGpus):
+      """
+      Set number of GPUs requested for submit run.
+      """
+      setError = False
+      if isinstance(nGpus,int):
+         self.nGpus = nGpus
+      else:
+         self.logger.log(logging.ERROR,getLogMessage("nGpus must be an integer"))
+         setError = True
+
+      return(setError)
+
+
+   def resetNgpus(self):
+      """
+      Set number of GPUs requested for submit run to the default value.
+      """
+      resetError = False
+      self.nGpus = None
+
+      return(resetError)
+
+
+   def __addNgpusArgs(self):
+      if self.nGpus:
+         self.args += ['--nGpus',str(self.nGpus)]
+
+
+   def __saveNgpus(self):
+      if self.nGpus:
+         self.savedCommand['nGpus'] = self.nGpus
+
+
+   def __loadNgpus(self):
+      self.resetNgpus()
+      if 'nGpus' in self.savedCommand:
+         self.setNgpus(self.savedCommand['nGpus'])
+
+
+   def setGPN(self,
+              gpn):
+      """
+      Set number of GPUs per node requested for submit run.
+      If number of cores per node is not set for the submit run
+      a default number of GPUs per node is set based on the
+      venue configuration.  This default value is typically based
+      on the node hardware.
+      """
+      setError = False
+      if isinstance(gpn,int):
+         self.gpn = gpn
+      else:
+         self.logger.log(logging.ERROR,getLogMessage("gpn must be an integer"))
+         setError = True
+
+      return(setError)
+
+
+   def resetGPN(self):
+      """
+      Set number of GPUs per node requested for submit run to the default value.
+      """
+      resetError = False
+      self.gpn = None
+
+      return(resetError)
+
+
+   def __addGPNArgs(self):
+      if self.gpn:
+         self.args += ['--gpn',str(self.gpn)]
+
+
+   def __saveGPN(self):
+      if self.gpn:
+         self.savedCommand['gpn'] = self.gpn
+
+
+   def __loadGPN(self):
+      self.resetGPN()
+      if 'gpn' in self.savedCommand:
+         self.setGPN(self.savedCommand['gpn'])
+
+
    def setNstripes(self,
                    nStripes):
       """
@@ -1026,6 +1159,47 @@ class SubmitCommand:
          self.setNstripes(self.savedCommand['nStripes'])
 
 
+   def setMemory(self,
+                 memoryMB):
+      """
+      Set memory requirement (MB) for submit run.
+      """
+      setError = False
+      if isinstance(memoryMB,int):
+         self.memoryMB = memoryMB
+      else:
+         self.logger.log(logging.ERROR,getLogMessage("memoryMB must be an integer"))
+         setError = True
+
+      return(setError)
+
+
+   def resetMemory(self):
+      """
+      Set memory requirement (MB) for submit run to the default value.
+      """
+      resetError = False
+      self.memoryMB = None
+
+      return(resetError)
+
+
+   def __addMemoryArgs(self):
+      if self.memoryMB:
+         self.args += ['--memory',str(self.memoryMB)]
+
+
+   def __saveMemory(self):
+      if self.memoryMB:
+         self.savedCommand['memoryMB'] = self.memoryMB
+
+
+   def __loadMemory(self):
+      self.resetMemory()
+      if 'memoryMB' in self.savedCommand:
+         self.setMemory(self.savedCommand['memoryMB'])
+
+
    def setWallTime(self,
                    wallTime):
       """
@@ -1833,6 +2007,134 @@ class SubmitCommand:
          self.setCommandArguments(self.savedCommand['commandArguments'])
 
 
+   def setJsonFiles(self,
+                    jsonFiles):
+      """
+      Supply set of json files containing submit arguments.
+      jsonFiles can be specified as a single filename or
+      of list of filenames.
+      """
+      setError = False
+      if   self.__isArgString(jsonFiles):
+         self.jsonFiles = [jsonFiles]
+      elif isinstance(jsonFiles,list):
+         self.jsonFiles = copy.copy(jsonFiles)
+      else:
+         self.logger.log(logging.ERROR,getLogMessage("jsonFiles must be a string or list"))
+         setError = True
+
+      return(setError)
+
+
+   def resetJsonFiles(self):
+      """
+      Remove jsonFiles from submit run.
+      """
+      resetError = False
+      self.jsonFiles = []
+
+      return(resetError)
+
+
+#  def __addJsonFilesArgs(self):
+#     if self.jsonFiles:
+#        for jsonFile in self.jsonFiles:
+#           self.args += ['--json',jsonFile]
+
+
+#  def __saveJsonFiles(self):
+#     if self.jsonFiles:
+#        self.savedCommand['jsonFiles'] = copy.copy(self.jsonFiles)
+
+
+   def __loadJsonFiles(self):
+      self.resetJsonFiles()
+      if 'jsonFiles' in self.savedCommand:
+         self.setJsonFiles(self.savedCommand['jsonFiles'])
+
+
+   def setSavejson(self,
+                   savejson):
+      """
+      Set file for saving submit command.
+      """
+      setError = False
+      if self.__isArgString(savejson):
+         self.savejson = savejson
+      else:
+         self.logger.log(logging.ERROR,getLogMessage("savejson must be a string"))
+         setError = True
+
+      return(setError)
+
+
+   def resetSavejson(self):
+      """
+      Set submit json file to default name.
+      """
+      resetError = False
+      self.savejson = ""
+
+      return(resetError)
+
+
+   def __addSavejsonArgs(self):
+      if self.savejson:
+         self.args += ['--savejson',self.savejson]
+
+
+   def __saveSavejson(self):
+      if self.savejson:
+         self.savedCommand['savejson'] = self.savejson
+
+
+   def __loadSavejson(self):
+      self.resetSavejson()
+      if 'savejson' in self.savedCommand:
+         self.setSavejson(self.savedCommand['savejson'])
+
+
+   def setShowCommand(self,
+                      showCommand=False):
+      """
+      Echo submit command to stdout.
+      """
+      setError = False
+      if isinstance(showCommand,bool):
+         self.showCommand = showCommand
+      else:
+         self.logger.log(logging.ERROR,getLogMessage("showCommand must be a boolean"))
+         setError = True
+
+      return(setError)
+
+
+   def resetShowCommand(self):
+      """
+      Remove showCommand.
+      """
+      resetError = False
+      self.showCommand = False
+
+      return(resetError)
+
+
+   def __addShowCommandArg(self):
+      if self.showCommand:
+         self.args += ['--show']
+
+
+   def __saveShowCommand(self):
+      if self.showCommand:
+         self.savedCommand['showCommand'] = self.showCommand
+
+
+   def __loadShowCommand(self):
+      self.resetShowCommand()
+      if 'showCommand' in self.savedCommand:
+         self.setShowCommand(self.savedCommand['showCommand'])
+
+
    def setStdin(self,
                 stdinPath):
       """
@@ -1860,7 +2162,8 @@ class SubmitCommand:
 
    def __saveStdin(self):
       if self.stdin:
-         self.savedCommand['stdin'] = self.stdin
+         if self.__isArgString(self.stdin):
+            self.savedCommand['stdin'] = self.stdin
 
 
    def __loadStdin(self):
@@ -1870,7 +2173,7 @@ class SubmitCommand:
 
 
    def __buildCommandArgumentsArgs(self,
-                                   args):
+                                   args=None):
       del self.args
       self.args = []
 
@@ -1888,6 +2191,8 @@ class SubmitCommand:
                self.__addQueryJobsArgs()
                self.__addKillJobsArgs()
                self.__addVenueStatusArgs()
+               self.__addShowCommandArg()
+               self.__addSavejsonArgs()
                self.__addReportMetricsArg()
                self.__addRedundancyArgs()
                self.__addRunNameArgs()
@@ -1895,9 +2200,13 @@ class SubmitCommand:
                   self.__addLocalArg()
                   self.__addNstripesArgs()
                else:
+                  self.__addAsynchronousArg()
                   self.__addVenuesArgs()
                   self.__addNcoresArgs()
                   self.__addPPNArgs()
+                  self.__addNgpusArgs()
+                  self.__addGPNArgs()
+                  self.__addMemoryArgs()
                   self.__addWallTimeArgs()
                self.__addParametersArgs()
                self.__addInputFilesArgs()
@@ -1920,39 +2229,46 @@ class SubmitCommand:
       """
       Reset submit command settings to default values.
       """
-      self.__resetHelp()
-      self.__resetVersion()
-      self.__resetDebug()
-      self.__resetLocal()
-      self.__resetQueryJobs()
-      self.__resetKillJobs()
-      self.__resetVenueStatus()
-      self.__resetVenues()
-      self.__resetInputFiles()
-      self.__resetDefaultSeparator()
-      self.__resetParameters()
-      self.__resetDataFile()
-      self.__resetNcores()
-      self.__resetPPN()
-      self.__resetNstripes()
-      self.__resetWallTime()
-      self.__resetEnvironmentVariables()
-      self.__resetRunName()
-      self.__resetManager()
-      self.__resetRedundancy()
-      self.__resetReportMetrics()
-      self.__resetDetach()
-      self.__resetAttachId()
-      self.__resetWait()
-      self.__resetQuota()
-      self.__resetDefaultTailNlines()
-      self.__resetTailStdout()
-      self.__resetTailStderr()
-      self.__resetTailFiles()
-      self.__resetProgress()
-      self.__resetCommand()
-      self.__resetCommandArguments()
-      self.__resetStdin()
+      self.resetHelp()
+      self.resetVersion()
+      self.resetDebug()
+      self.resetLocal()
+      self.resetAsynchronous()
+      self.resetQueryJobs()
+      self.resetKillJobs()
+      self.resetVenueStatus()
+      self.resetVenues()
+      self.resetInputFiles()
+      self.resetJsonFiles()
+      self.resetSavejson()
+      self.resetShowCommand()
+      self.resetDefaultSeparator()
+      self.resetParameters()
+      self.resetDataFile()
+      self.resetNcores()
+      self.resetPPN()
+      self.resetNgpus()
+      self.resetGPN()
+      self.resetNstripes()
+      self.resetMemory()
+      self.resetWallTime()
+      self.resetEnvironmentVariables()
+      self.resetRunName()
+      self.resetManager()
+      self.resetRedundancy()
+      self.resetReportMetrics()
+      self.resetDetach()
+      self.resetAttachId()
+      self.resetWait()
+      self.resetQuota()
+      self.resetDefaultTailNlines()
+      self.resetTailStdout()
+      self.resetTailStderr()
+      self.resetTailFiles()
+      self.resetProgress()
+      self.resetCommand()
+      self.resetCommandArguments()
+      self.resetStdin()
 
 
    def saveSubmitCommand(self,
@@ -1970,6 +2286,7 @@ class SubmitCommand:
       self.__saveVersion()
       self.__saveDebug()
       self.__saveLocal()
+      self.__saveAsynchronous()
       self.__saveQueryJobs()
       self.__saveKillJobs()
       self.__saveVenueStatus()
@@ -1980,7 +2297,10 @@ class SubmitCommand:
       self.__saveDataFile()
       self.__saveNcores()
       self.__savePPN()
+      self.__saveNgpus()
+      self.__saveGPN()
       self.__saveNstripes()
+      self.__saveMemory()
       self.__saveWallTime()
       self.__saveEnvironmentVariables()
       self.__saveRunName()
@@ -1996,9 +2316,11 @@ class SubmitCommand:
       self.__saveTailStderr()
       self.__saveTailFiles()
       self.__saveProgress()
+      self.__saveShowCommand()
+      self.__saveSavejson()
+      self.__saveStdin()
       self.__saveCommand()
       self.__saveCommandArguments()
-      self.__saveStdin()
 
       jsonMessage = json.dumps(self.savedCommand,indent=indent,sort_keys=sortKeys) + '\n'
       if submitCommandJSONFile:
@@ -2049,17 +2371,24 @@ class SubmitCommand:
          self.__loadVersion()
          self.__loadDebug()
          self.__loadLocal()
+         self.__loadAsynchronous()
          self.__loadQueryJobs()
          self.__loadKillJobs()
          self.__loadVenueStatus()
          self.__loadVenues()
          self.__loadInputFiles()
+         self.__loadJsonFiles()
+         self.__loadSavejson()
+         self.__loadShowCommand()
          self.__loadDefaultSeparator()
          self.__loadParameters()
          self.__loadDataFile()
          self.__loadNcores()
          self.__loadPPN()
+         self.__loadNgpus()
+         self.__loadGPN()
          self.__loadNstripes()
+         self.__loadMemory()
          self.__loadWallTime()
          self.__loadEnvironmentVariables()
          self.__loadRunName()
@@ -2082,6 +2411,38 @@ class SubmitCommand:
       return(loadError)
 
 
+   def __show(self,
+              textWidth=80):
+      """
+      Show submit command as determined from previous settings.
+      """
+      if self.args:
+         command = ['submit'] + self.args
+      else:
+         command = ['submit']
+
+      commandLine = ' '.join(command)
+      if sys.stdin:
+         try:
+            stdinPath = sys.stdin.name
+            if stdinPath == "<stdin>":
+               stdinFd = sys.stdin.fileno()
+               pid = os.getpid()
+               stdinLink = os.path.join(os.sep,"proc",str(pid),"fd",str(stdinFd))
+               stdinPath = os.readlink(stdinLink)
+               if stdinPath.startswith(os.path.join(os.sep,"dev","pts") + os.sep):
+                  stdinPath = None
+         except:
+            stdinPath = None
+         else:
+            if stdinPath:
+               commandLine += " < " + stdinPath
+
+      prettyCommand = textwrap.fill(commandLine,textWidth,subsequent_indent='       ',
+                                    break_long_words=False,break_on_hyphens=False)
+      self.__writeToStdout(prettyCommand + '\n')
+
+
    def show(self,
             args=None,
             textWidth=80):
@@ -2091,16 +2452,46 @@ class SubmitCommand:
       replaced or overwritten.
       """
       self.__buildCommandArgumentsArgs(args)
+      self.__show(textWidth)
 
-      if self.args:
-         command = ['submit'] + self.args
+
+   def __mergeJsonInputs(self,
+                         args):
+      """
+      Merge arguments specified in JSON files.
+      """
+
+      userStdin = self.stdin
+      if userStdin is None:
+         userStdin = sys.stdin
+      if args:
+         mergedArgs = []
+         self.resetSubmitCommand()
+         while True:
+            try:
+               jsonIndex = args.index("--json")
+            except:
+               mergedArgs += args
+               break
+            else:
+               self.loadSubmitCommand(args[jsonIndex+1])
+               self.__buildCommandArgumentsArgs()
+               mergedArgs += args[:jsonIndex] + self.args
+               args = args[jsonIndex+2:]
+               if self.stdin:
+                  userStdin = self.stdin
+               self.resetSubmitCommand()
+               del self.args
+               self.args = []
+
+         del self.savedCommand
+         self.savedCommand = {}
       else:
-         command = ['submit']
+         mergedArgs = None
 
-      commandLine = ' '.join(command)
-      prettyCommand = textwrap.fill(commandLine,textWidth,subsequent_indent='       ',
-                                    break_long_words=False,break_on_hyphens=False)
-      self.__writeToStdout(prettyCommand + '\n')
+      self.stdin = userStdin
+
+      return mergedArgs
 
 
    def submit(self,
@@ -2114,12 +2505,11 @@ class SubmitCommand:
       result = None
       exitCode = 0
 
+      mergedArgs = self.__mergeJsonInputs(args)
+
       userStdin = stdin
       if not userStdin:
-         if args:
-            userStdin = sys.stdin
-         else:
-            userStdin = self.stdin
+         userStdin = self.stdin
       if not userStdin:
          userStdin = os.devnull
 
@@ -2181,7 +2571,15 @@ class SubmitCommand:
          sys.stdin = submitStdin
          self.submitClient = SubmitClient(self.configFilePath)
 
-         self.__buildCommandArgumentsArgs(args)
+         self.__buildCommandArgumentsArgs(mergedArgs)
+
+         try:
+            showCommandIndex = self.args.index("--show")
+         except:
+            pass
+         else:
+            self.__show(textWidth=80)
+#           del self.args[showCommandIndex:showCommandIndex+1]
 
          result = self.submitClient.executeSubmission(self.instantiatorPath,
                                                       self.args)
